# -*- coding: utf-8 -*-
"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

from collections import deque

nil = object()  # used to distinguish from None


class TrieNode(object):
    """
    Node of trie/Aho-Corasick automaton
    """

    __slots__ = ['char', 'output', 'fail', 'children']

    def __init__(self, char):
        """
        Constructs an empty node
        """

        self.char = char  # character
        self.output = nil  # an output function for this node
        self.fail = nil  # fail link used by Aho-Corasick automaton
        self.children = {}  # children

    def __repr__(self):
        """
        Textual representation of node.
        """

        if self.output is not nil:
            return "<TrieNode '%s' '%s'>" % (self.char, self.output)
        else:
            return "<TrieNode '%s'>" % self.char


class Trie(object):
    """
    Trie/Aho-Corasick automaton.
    """

    def __init__(self):
        """
        Construct an empty trie
        """

        self.root = TrieNode('')

    def __get_node(self, word):
        """
        Private function retrieving a final node of trie
        for given word

        Returns node or None, if the trie doesn't contain the word.
        """

        node = self.root
        for c in word:
            try:
                node = node.children[c]
            except KeyError:
                return None

        return node

    def get(self, word, default=nil):
        """
        Retrieves output value associated with word.

        If there is no word returns default value,
        and if default is not given rises KeyError.
        """

        node = self.__get_node(word)
        output = nil
        if node:
            output = node.output

        if output is nil:
            if default is nil:
                raise KeyError("no key '%s'" % word)
            else:
                return default
        else:
            return output

    def keys(self):
        """
        Generator returning all keys (i.e. word) stored in trie
        """

        for key, _ in self.items():
            yield key

    def values(self):
        """
        Generator returning all values associated with words stored in a trie.
        """

        for _, value in self.items():
            yield value

    def items(self):
        """
        Generator returning all keys and values stored in a trie.
        """

        L = []

        def aux(node, s):
            s = s + node.char
            if node.output is not nil:
                L.append((s, node.output))

            for child in node.children.values():
                if child is not node:
                    aux(child, s)

        aux(self.root, '')
        return iter(L)

    def __len__(self):
        """
        Calculates number of words in a trie.
        """

        stack = deque()
        stack.append(self.root)
        n = 0
        while stack:
            node = stack.pop()
            if node.output is not nil:
                n += 1

            for child in node.children.values():
                stack.append(child)

        return n

    def add_word(self, word, value):
        """
        Adds word and associated value.

        If word already exists, its value is replaced.
        """
        if not word:
            return

        node = self.root
        for c in word:
            try:
                node = node.children[c]
            except KeyError:
                n = TrieNode(c)
                node.children[c] = n
                node = n

        node.output = value

    def clear(self):
        """
        Clears trie.
        """

        self.root = TrieNode('')

    def exists(self, word):
        """
        Checks if whole word is present in the trie.
        """

        node = self.__get_node(word)
        if node:
            return bool(node.output != nil)
        else:
            return False

    def match(self, word):
        """
        Checks if word is a prefix of any existing word in the trie.
        """

        return (self.__get_node(word) is not None)

    def make_automaton(self):
        """
        Converts trie to Aho-Corasick automaton.
        """

        queue = deque()

        # 1.
        for i in range(256):
            c = chr(i)
            if c in self.root.children:
                node = self.root.children[c]
                node.fail = self.root  # f(s) = 0
                queue.append(node)
            else:
                self.root.children[c] = self.root

        # 2.
        while queue:
            r = queue.popleft()
            for node in r.children.values():
                queue.append(node)
                state = r.fail
                while node.char not in state.children:
                        state = state.fail

                node.fail = state.children.get(node.char, self.root)

    def iter(self, string):
        """
        Generator performs Aho-Corasick search string algorithm, yielding
        tuples containing two values:
        - position in string
        - outputs associated with matched strings
        """
        state = self.root
        for index, c in enumerate(string):
            while c not in state.children:
                state = state.fail

            state = state.children.get(c, self.root)

            tmp = state
            output = []
            while tmp is not nil:
                if tmp.output is not nil:
                    output.append(tmp.output)

                tmp = tmp.fail

            if output:
                yield (index, output)

    def iter_long(self, string):
        """
        Generator performs a modified Aho-Corasick search string algorithm,
        which maches only the longest word.

        """
        state = self.root
        last = None

        index = 0
        while index < len(string):
            c = string[index]

            if c in state.children:
                state = state.children[c]

                if state.output is not nil:
                    # save the last node on the path
                    last = (state.output, index)

                index += 1
            else:
                if last:
                    # return the saved match
                    yield last

                    # and start over, as we don't want overlapped results
                    # Note: this leads to quadratic complexity in the worst case
                    index = last[1] + 1
                    state = self.root
                    last = None
                else:
                    # if no output, perform classic Aho-Corasick algorithm
                    while c not in state.children:
                        state = state.fail

        # corner case
        if last:
            yield last

    def find_all(self, string, callback):
        """
        Wrapper on iter method, callback gets an iterator result
        """
        for index, output in self.iter(string):
            callback(index, output)


if __name__ == '__main__':

    def demo():
        words = "he hers his she hi him man".split()

        t = Trie();
        for w in words:
            t.add_word(w, w)

        s = "he rshershidamanza "

        t.make_automaton()
        for res in t.items():
            print(res)

        for res in t.iter(s):
            print
            print('%s' % s)
            pos, matches = res
            for fragment in matches:
                print('%s%s' % ((pos - len(fragment) + 1) * ' ', fragment))

    demo()

    def bug():
        patterns = ['GT-C3303', 'SAMSUNG-GT-C3303K/']
        text = 'SAMSUNG-GT-C3303i/1.0 NetFront/3.5 Profile/MIDP-2.0 Configuration/CLDC-1.1'

        t = Trie()
        for pattern in patterns:
            ret = t.add_word(pattern, (0, pattern))

        t.make_automaton()

        res = list(t.iter(text))

        assert len(res) == 1, 'failed'

    bug()

# vim: ts=4 sw=4 nowrap



# -*- coding: utf-8 -*-
"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import os
import unittest
from pathlib import Path

import ahocorasick

from pytestingutils import conv
from pytestingutils import on_linux


def build_automaton():
    ac = ahocorasick.Automaton()
    ac.add_word(conv('SSSSS'), 1)
    ac.make_automaton()
    return ac


def get_memory_usage():
    # Linux only
    pid = os.getpid()

    lines = []
    try:
        with open('/proc/%d/status' % pid, 'rt') as f:
            lines = f.readlines()
    except:
        pass

    for line in lines:
        if line.startswith('VmSize'):
            return float(line.split()[1])

    return 0


@unittest.skipIf(not on_linux, "Works only on linux")
class MemoryUsageDoesNotGrow(unittest.TestCase):

    def test_memory_usage_does_not_grow(self):

        ac = build_automaton()
        
        here = Path(__file__).parent
        with open(here.parent / 'README.rst') as f:
            data = f.read()[:1024 * 2]
            data = conv(data)

        before = get_memory_usage()

        for _ in range(1000):
            for start in range(0, len(data) - 20):
                ac.iter(data, start)

        after = get_memory_usage()
        assert before == after



# -*- coding: utf-8 -*-
"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import pickle

import ahocorasick
from pytestingutils import conv


def test_issue_26_pickle_workds():
    a = ahocorasick.Automaton(ahocorasick.STORE_INTS)
    a.add_word(conv('abc'), 12)
    a.make_automaton()
    pickle.dumps(a)



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""
import ahocorasick

from pytestingutils import conv


def iter_results(teststr, automaton):
    r = []
    for x in automaton.iter(teststr):
        r.append(x)

    return r


def find_all_results(teststr, automaton):
    r = []

    def append(x, teststr):
        r.append((x, teststr))

    automaton.find_all(teststr, append)
    return r


def test_issue56():
    automaton = ahocorasick.Automaton()

    for word in ("poke", "go", "pokegois", "egoist"):
        automaton.add_word(conv(word), word)

    automaton.make_automaton()

    teststr = conv("pokego pokego  pokegoist")
    expected = iter_results(teststr, automaton)
    findall = find_all_results(teststr, automaton)

    assert findall == expected




"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import ahocorasick
import os

from ahocorasick import EMPTY, TRIE


def dump2dot(automaton, file):

    def writeln(text=""):
        file.write(text + "\n")

    def nodename(nodeid):
        return 'node%x' % (nodeid & 0xffffffff)

    if automaton.kind == EMPTY:
        writeln("digraph empty {}")
        return

    if automaton.kind == TRIE:
        name = "trie"
    else:
        name = "ahocorasick"


    writeln("digraph %s {" % name)

    nodes, edges, fail = automaton.dump()

    # nodes
    for nodeid, end in nodes:
        if end:
            attr = '[shape=doublecircle, label=""]'
        else:
            attr = '[shape=circle, label=""]'

        writeln("\t%s %s" % (nodename(nodeid), attr))

    def format_label(label):
        label = str(label, 'ascii')
        label = label.replace('"', r'\"')
        return '"%s"' % label

    # trie edges
    for nodeid, label, destid in edges:
        writeln("\t%s -> %s [label=%s]" % (nodename(nodeid), nodename(destid), format_label(label)))

    # fail links
    for nodeid, failid in fail:
        writeln("\t%s -> %s [color=blue]" % (nodename(nodeid), nodename(failid)))

    writeln("}")


def show(automaton):
    path = '/dev/shm/%s.dot' % os.getpid()
    with open(path, 'wt') as f:
        dump2dot(automaton, f)

    os.system("xdot %s" % path)
    #os.system("dotty %s" % path)
    os.unlink(path)


if __name__ == '__main__':
    A = ahocorasick.Automaton(ahocorasick.STORE_LENGTH)

    A.add_word("he")
    A.add_word("her")
    A.add_word("hers")
    A.add_word("she")
    A.add_word("cat")
    A.add_word("shield")

    with open('trie.dot', 'wt') as f:
        dump2dot(A, f)

    A.make_automaton()
    with open('ahocorasick.dot', 'wt') as f:
        dump2dot(A, f)



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import random

from optparse import OptionParser


def main():
    options = parse_args()
    app = TestApplication(options)
    app.run()


chars = 'abcdefghijklmnopqestuvwxyzABCDEFGHIJKLMNOPQESTUVWXYZ0123456789.,;:-'


class TestApplication(object):

    def __init__(self, options):
        self.options = options

        random.seed(options.seed)

    def run(self):
        n = self.options.words

        for _i in range(n):
            self.generate_random_word()

    def generate_random_word(self):
        n = random.randint(1, self.options.maxlength + 1)
        s = ''
        for _i in range(n):
            s += random.choice(chars)

        return s


def parse_args():

    parser = OptionParser()
    parser.add_option(
        "--max-words", dest='words', type=int, default=50000, metavar='N',
        help="maximum number of words generated/loaded"
    )

    parser.add_option(
        "--random", dest='random', action='store_true', default=False,
        help="generate random words"
    )

    parser.add_option(
        "--seed", dest='seed', type=int, default=0, metavar='INT',
        help="random seed"
    )

    parser.add_option(
        "--random-max-len", dest='maxlength', type=int, default=100, metavar='K',
        help="maximum count of characters in a word"
    )

    (options, _rest) = parser.parse_args()

    return options


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import os
import random
import gzip
import pickle
import time

import ahocorasick

from optparse import OptionParser


def main():
    options = parse_args()
    app = TestApplication(options)
    app.run()


chars = 'abcdefghijklmnopqestuvwxyzABCDEFGHIJKLMNOPQESTUVWXYZ0123456789.,;:-'


class TestApplication(object):

    def __init__(self, options):
        self.options = options
        self.words = set()

        random.seed(options.seed)

    def run(self):
        self.A = ahocorasick.Automaton()

        if self.options.compare and (not self.options.pickle and not self.options.save):
            self.generate_words()

        if self.options.pickle or self.options.save:
            self.add_words()

        if self.options.pickle:
            t1 = time.time()
            self.pickle()
            t2 = time.time()
            print("   time: %0.2fs" % (t2 - t1))
            self.A.clear()

        if self.options.save:
            t1 = time.time()
            self.save()
            t2 = time.time()
            print("   time: %0.2fs" % (t2 - t1))
            self.A.clear()

        if self.options.unpickle:
            t1 = time.time()
            self.unpickle()
            t2 = time.time()
            print("   time: %0.2fs" % (t2 - t1))

        if self.options.load:
            t1 = time.time()
            self.load()
            t2 = time.time()
            print("   time: %0.2fs" % (t2 - t1))

        if self.options.compare:
            self.compare()

    def add_words(self):
        if self.options.random:
            self.__add_random_words()
        else:
            self.__add_from_file()

        print("Automaton statistics:")
        d = self.A.get_stats()
        print("- nodes_count  : %d" % d['nodes_count'])
        print("- words_count  : %d" % d['words_count'])
        print("- links_count  : %d" % d['links_count'])
        print("- longest_word : %d" % d['longest_word'])
        print("- sizeof_node  : %d" % d['sizeof_node'])
        print("- total_size   : %d" % d['total_size'])

    def __add_random_words(self):
        n = self.options.words

        print("Adding %d words" % n)
        while n > 0:
            word = self.generate_random_word()
            if self.options.compare:
                self.words.add(word)

            if self.A.add_word(word, True):
                n -= 1

    def __add_from_file(self):
        n = self.options.words

        print("Adding %d words from %s" % (n, self.options.file_gz))
        for i, word in enumerate(self.read()):
            if i > n:
                return

            if self.options.compare:
                self.words.add(word)

            self.A.add_word(word, True)

    def generate_words(self):
        if self.options.random:
            self.__generate_random_words()
        else:
            self.__load_words()

    def __generate_random_words(self):
        n = self.options.words

        print("Generating %d words" % n)
        while len(self.words) < n:
            word = self.generate_random_word()
            self.words.add(word)

    def __load_words(self):
        n = self.options.words
        print ("Loading %d words from %s" % (n, self.options.file_gz))
        for i, word in enumerate(self.read()):
            if i < n:
                self.words.add(word)
            else:
                return

    def read(self):
        with gzip.open(self.options.file_gz, "rt", encoding="utf-8") as f:
            for line in f:
                yield line.strip()

    def pickle(self):
        path = self.options.picklepath

        print("Pickling automaton in %s" % path)

        with open(path, 'wb') as f:
            pickle.dump(self.A, f)

        size = os.path.getsize(path)
        print("   file size is %s" % format_size(size))

    def unpickle(self):
        path = self.options.picklepath

        print("Unpickling automaton from %s" % path)
        with open(path, 'rb') as f:
            self.A = pickle.load(f)

    def save(self):
        path = self.options.picklepath

        print("Saving automaton in %s" % path)

        self.A.save(path, pickle.dumps);

        size = os.path.getsize(path)
        print("   file size is %s" % format_size(size))

    def load(self):
        path = self.options.picklepath

        print("Loading automaton from %s" % path)

        self.A = ahocorasick.load(path, pickle.loads)

    def compare(self):
        print("Comparing added words with restored automaton")

        for word in self.A:
            self.words.remove(word)

        if self.words:
            print("Not all words were restored (%d missing)" % len(self.words))

    def generate_random_word(self):
        n = random.randint(1, self.options.maxlength + 1)
        s = ''
        for i in range(n):
            s += random.choice(chars)

        return s


def format_size(size):
    units = [
        ('GB', 1024 ** 3),
        ('MB', 1024 ** 2),
        ('kB', 1024),
    ]

    for suffix, threshold in units:
        if size > threshold:
            return '%0.2f %s (%d bytes)' % (float(size) / threshold, suffix, size)

    return '%d bytes' % size


def parse_args():

    parser = OptionParser()
    parser.add_option(
        "--pickle-path", dest='picklepath', default='pickle_stresstest.pickle',
        help="path used in pickling/unpickling"
    )

    parser.add_option(
        "-p", "--pickle", dest='pickle', action='store_true', default=False,
        help="perform pickle operation on generated/loaded words"
    )

    parser.add_option(
        "-u", "--unpickle", dest='unpickle', action='store_true', default=False,
        help="perform unpickle operation on previously pickled data"
    )

    parser.add_option(
        "-s", "--save", dest='save', action='store_true', default=False,
        help="perform save operation on generated/loaded words"
    )

    parser.add_option(
        "-l", "--load", dest='load', action='store_true', default=False,
        help="perform load operation on previously saved data"
    )

    parser.add_option(
        "-c", "--compare", action='store_true', default=False,
        help="compare generated/loaded words with unpickled data"
    )

    parser.add_option(
        "--max-words", dest='words', type=int, default=50000, metavar='N',
        help="maximum number of words generated/loaded"
    )

    parser.add_option(
        "--random", dest='random', action='store_true', default=False,
        help="generate random words"
    )

    parser.add_option(
        "--seed", dest='seed', type=int, default=0, metavar='INT',
        help="random seed"
    )

    parser.add_option(
        "--random-max-len", dest='maxlength', type=int, default=100, metavar='K',
        help="maximum count of characters in a word"
    )

    parser.add_option(
        "--file-gz", metavar='FILE',
        help="load words from utf8-encoded gz file"
    )

    (options, rest) = parser.parse_args()

    if not (options.file_gz or options.random):
        raise parser.error("pass --random or --file-gz option")

    if (options.pickle or options.unpickle) and (options.save or options.load):
        raise parser.error("use separately --pickle/--unpickle and --save/--load")

    return options


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import sys


def main():
    try:
        path = sys.argv[1]
    except IndexError:
        path = 'memory.dump'

    app = Application(path)
    if app.run():
        sys.exit(0)
    else:
        sys.exit(1)


class Application(object):

    def __init__(self, path):
        self.path = path
        self.memory = {}

    def run(self):
        with open(self.path, 'rt') as f:
            self.analyze(f)

        self.print_leaks()

        return len(self.memory) == 0

    def analyze(self, file):
        self.memory = {}

        for i, line in enumerate(file):
            fields = line.split()
            action = fields[0]
            if action == 'A':
                id = fields[1]
                addr = fields[2]
                size = int(fields[3])

                assert addr not in self.memory
                self.memory[addr] = (id, size)

            elif action == 'R':

                id = fields[1]
                oldaddr = fields[2]
                newaddr = fields[3]
                size = int(fields[4])

                try:
                    key = int(oldaddr, 16)
                    del self.memory[oldaddr]
                except ValueError:
                    pass

                assert newaddr not in self.memory
                self.memory[newaddr] = (id, size)

            elif action == 'F':

                addr = fields[1]
                if addr in self.memory:
                    del self.memory[addr]

    def print_leaks(self):
        n = len(self.memory)
        if n == 0:
            return

        print('There are %d leaks:' % n)
        tmp = [(int(id), addr, size) for addr, (id, size) in self.memory.items()]
        tmp.sort(key=lambda item: item[0])

        for id, addr, size in tmp:
            print('#%s: %s %d' % (id, addr, size))


if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

from unittest.case import skipIf

import ahocorasick


@skipIf(not ahocorasick.unicode, "Run only with unicode build")
def test_issue_53_unicode():
    auto = ahocorasick.Automaton()
    auto.add_word('wounded', 'wounded')

    auto.make_automaton()

    results = list(auto.iter('Winning \U0001F629 so gutted, can\'t do anything for 4 weeks... Myth. #wounded'))
    expected = [(67, 'wounded')]
    assert results == expected

    expected = [(65, 'wounded')]
    results = list(auto.iter('Winning so gutted, can\'t do anything for 4 weeks... Myth. #wounded'))
    assert results == expected


@skipIf(ahocorasick.unicode, "Run only with bytes build")
def test_issue_53_bytes():
    auto = ahocorasick.Automaton()
    auto.add_word(b'wounded', b'wounded')

    auto.make_automaton()

    results = list(auto.iter('Winning \U0001F629 so gutted, can\'t do anything for 4 weeks... Myth. #wounded'.encode(encoding='utf_8')))
    expected = [(70, b'wounded')]
    assert results == expected

    expected = [(65, b'wounded')]
    results = list(auto.iter(b'Winning so gutted, can\'t do anything for 4 weeks... Myth. #wounded'))
    assert results == expected



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import os
import sys


def main():
    app = Application(sys.argv[1], sys.argv[2])
    if app.run():
        sys.exit(0)
    else:
        sys.exit(1)


class Application(object):

    def __init__(self, srcdir, path):
        self.srcdir = srcdir
        self.path = path
        self.sources = set()
        self.dump = 0

    def run(self):
        self.gather_sources()

        with open(self.path, 'rt') as f:
            leaks = self.analyze(f)

        if not leaks:
            return True
        else:
            print("Following references found in %s (file -> line no)" % self.path)
            for name in sorted(leaks):
                lines = ', '.join(map(str, leaks[name]))
                print("- %s: %s" % (name, lines))

    def gather_sources(self):
        for path in os.listdir(self.srcdir):
            if path.endswith('.c'):
                self.sources.add(path)

    def analyze(self, file):
        result = {}
        for k, line in enumerate(file):
            if 'by 0x' in line or 'at 0x' in line:
                try:
                    # by 0xfffff: function (file.c:1234)
                    #                       ^^^^^^
                    index = line.rindex('(') + 1
                    name = line[index:]
                    if name.startswith('in '):
                        continue

                    index = name.index(':')
                    name = name[:index]

                    if self.dump:
                        print(name)
                except ValueError:
                    continue

                if name in self.sources:
                    if name not in result:
                        result[name] = []

                    result[name].append(k)

        return result


if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-
"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import ahocorasick
from pytestingutils import conv


def test_issue_10_case_1():
    ac = ahocorasick.Automaton()
    ac.add_word(conv('S'), 1)
    ac.make_automaton()
    buffer = conv('SSS')
    count = 0
    for _item in ac.iter(buffer, 0, 3):  # this causes an error
        count += 1

    assert count == 3


def test_issue_10_case_2():
    ac = ahocorasick.Automaton()
    ac.add_word(conv('S'), 1)
    ac.make_automaton()
    buffer = conv('SSS')
    count = 0
    for _item in ac.iter(buffer, 0, 2):  # no error, but it misses the last 'S' in the buffer
        count += 1

    assert count == 2



# -*- coding: utf-8 -*-
"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import ahocorasick

from pytestingutils import conv


def test_issue_5():
    A = ahocorasick.Automaton()

    # add some words to trie
    for index, word in enumerate(conv("he her hers she").split()):
        A.add_word(word, (index, word))

    A = None  #### segfault here



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import random
import gzip

import ahocorasick

from optparse import OptionParser


def main():
    options = parse_args()
    app = TestApplication(options)
    app.run()


chars = 'abcdefghijklmnopqestuvwxyzABCDEFGHIJKLMNOPQESTUVWXYZ0123456789.,;:-'


class TestApplication(object):

    def __init__(self, options):
        self.options = options
        self.words = []

        random.seed(options.seed)

    def run(self):
        self.A = ahocorasick.Automaton()

        self.add_words()
        self.remove()

    def add_words(self):
        if self.options.random:
            self.__add_random_words()
        else:
            self.__add_from_file()

        print("Automaton statistics:")
        d = self.A.get_stats()
        print("- nodes_count  : %d" % d['nodes_count'])
        print("- words_count  : %d" % d['words_count'])
        print("- links_count  : %d" % d['links_count'])
        print("- longest_word : %d" % d['longest_word'])
        print("- sizeof_node  : %d" % d['sizeof_node'])
        print("- total_size   : %d" % d['total_size'])

    def remove(self):
        print("Removing %d words" % len(self.words))
        random.shuffle(self.words)
        for word in self.words:
            self.A.remove_word(word)

        print("Automaton statistics:")
        d = self.A.get_stats()
        print("- nodes_count  : %d" % d['nodes_count'])
        print("- words_count  : %d" % d['words_count'])
        print("- links_count  : %d" % d['links_count'])
        print("- longest_word : %d" % d['longest_word'])
        print("- sizeof_node  : %d" % d['sizeof_node'])
        print("- total_size   : %d" % d['total_size'])

    def __add_random_words(self):
        n = self.options.words

        print("Adding %d words" % n)
        while n > 0:
            word = self.generate_random_word()
            if self.A.add_word(word, True):
                n -= 1
                self.words.append(word)

    def __add_from_file(self):
        n = self.options.words

        print("Adding %d words from %s" % (n, self.options.file_gz))
        for i, word in enumerate(self.read()):
            if i > n:
                return

            self.A.add_word(word, True)
            self.words.append(word)

    def generate_words(self):
        if self.options.random:
            self.__generate_random_words()
        else:
            self.__load_words()

    def __generate_random_words(self):
        n = self.options.words

        print("Generating %d words" % n)
        while len(self.words) < n:
            word = self.generate_random_word()
            self.words.add(word)

    def __load_words(self):
        n = self.options.words
        print ("Loading %d words from %s" % (n, self.options.file_gz))
        for i, word in enumerate(self.read()):
            if i < n:
                self.words.add(word)
            else:
                return

    def read(self):
        with gzip.open(self.options.file_gz, "rt", encoding="utf-8") as f:
            for line in f:
                yield line.strip()

    def generate_random_word(self):
        n = random.randint(1, self.options.maxlength + 1)
        s = ''
        for i in range(n):
            s += random.choice(chars)

        return s


def format_size(size):
    units = [
        ('GB', 1024 ** 3),
        ('MB', 1024 ** 2),
        ('kB', 1024),
    ]

    for suffix, threshold in units:
        if size > threshold:
            return '%0.2f %s (%d bytes)' % (float(size) / threshold, suffix, size)

    return '%d bytes' % size


def parse_args():

    parser = OptionParser()
    parser.add_option(
        "--max-words", dest='words', type=int, default=50000, metavar='N',
        help="maximum number of words generated/loaded"
    )

    parser.add_option(
        "--random", dest='random', action='store_true', default=False,
        help="generate random words"
    )

    parser.add_option(
        "--seed", dest='seed', type=int, default=0, metavar='INT',
        help="random seed"
    )

    parser.add_option(
        "--random-max-len", dest='maxlength', type=int, default=100, metavar='K',
        help="maximum count of characters in a word"
    )

    parser.add_option(
        "--file-gz", metavar='FILE',
        help="load words from utf8-encoded gz file"
    )

    (options, rest) = parser.parse_args()

    if not (options.file_gz or options.random):
        raise parser.error("pass --random or --file-gz option")

    return options


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import os
from pathlib import Path

import pytest

import ahocorasick

from pytestingutils import conv
from pytestingutils import on_linux


def get_memory_usage():
    # Linux only
    pid = os.getpid()

    lines = []
    try:
        with open('/proc/%d/status' % pid, 'rt') as f:
            lines = f.readlines()
    except:
        pass

    for line in lines:
        if line.startswith('VmSize'):
            return float(line.split()[1])

    return 0


def build_automaton():
    here = Path(__file__).parent
    with open(here.parent.parent / 'README.rst') as f:
        data = f.read()
        data = conv(data)
        data = data.split()

    ac = ahocorasick.Automaton()
    for i, word in enumerate(data):
        ac.add_word(word, i)

    ac.make_automaton()
    return ac


@pytest.mark.skipif(not on_linux, reason="Works only on linux")
def does_not_leak_memory():
    # This is problematic, some memory leaks are still there
    # so this needs to be investigated and re-enabled after fixing
    # See issues:
    # https://github.com/WojciechMula/pyahocorasick/issues/135
    # https://github.com/WojciechMula/pyahocorasick/issues/198
    ac = build_automaton()
    before = get_memory_usage()

    for i in range(1024):
        s = list(ac.keys())

    after = get_memory_usage()
    assert before == after



"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import unittest
from pyahocorasick import Trie

class TestTrie(unittest.TestCase):
    def testEmptyTrieShouldNotContainsAnyWords(self):
        t = Trie()
        self.assertEqual(len(t), 0)


    def testAddedWordShouldBeCountedAndAvailableForRetrieval(self):
        t = Trie()
        t.add_word('python', 'value')
        self.assertEqual(len(t), 1)
        self.assertEqual(t.get('python'), 'value')


    def testAddingExistingWordShouldReplaceAssociatedValue(self):
        t = Trie()
        t.add_word('python', 'value')
        self.assertEqual(len(t), 1)
        self.assertEqual(t.get('python'), 'value')

        t.add_word('python', 'other')
        self.assertEqual(len(t), 1)
        self.assertEqual(t.get('python'), 'other')

    def testGetUnknowWordWithoutDefaultValueShouldRaiseException(self):
        t = Trie()
        with self.assertRaises(KeyError):
            t.get('python')


    def testGetUnknowWordWithDefaultValueShouldReturnDefault(self):
        t = Trie()
        self.assertEqual(t.get('python', 'default'), 'default')


    def testExistShouldDetectAddedWords(self):
        t = Trie()
        t.add_word('python', 'value')
        t.add_word('ada', 'value')

        self.assertTrue(t.exists('python'))
        self.assertTrue(t.exists('ada'))


    def testExistShouldReturnFailOnUnknownWord(self):
        t = Trie()
        t.add_word('python', 'value')

        self.assertFalse(t.exists('ada'))


    def testMatchShouldDetecAllPrefixesIncludingWord(self):
        t = Trie()
        t.add_word('python', 'value')
        t.add_word('ada', 'value')

        self.assertTrue(t.match('a'))
        self.assertTrue(t.match('ad'))
        self.assertTrue(t.match('ada'))

        self.assertTrue(t.match('p'))
        self.assertTrue(t.match('py'))
        self.assertTrue(t.match('pyt'))
        self.assertTrue(t.match('pyth'))
        self.assertTrue(t.match('pytho'))
        self.assertTrue(t.match('python'))


    def testItemsShouldReturnAllItemsAlreadyAddedToTheTrie(self):
        t = Trie()

        t.add_word('python', 1)
        t.add_word('ada', 2)
        t.add_word('perl', 3)
        t.add_word('pascal', 4)
        t.add_word('php', 5)

        result = list(t.items())
        self.assertEqual(len(result), 5)
        self.assertIn(('python', 1), result)
        self.assertIn(('ada',    2), result)
        self.assertIn(('perl',   3), result)
        self.assertIn(('pascal', 4), result)
        self.assertIn(('php',    5), result)


    def testKeysShouldReturnAllKeysAlreadyAddedToTheTrie(self):
        t = Trie()

        t.add_word('python', 1)
        t.add_word('ada', 2)
        t.add_word('perl', 3)
        t.add_word('pascal', 4)
        t.add_word('php', 5)

        result = list(t.keys())
        self.assertEqual(len(result), 5)
        self.assertIn('python',result)
        self.assertIn('ada',   result)
        self.assertIn('perl',  result)
        self.assertIn('pascal',result)
        self.assertIn('php',   result)


    def testValuesShouldReturnAllValuesAlreadyAddedToTheTrie(self):
        t = Trie()

        t.add_word('python', 1)
        t.add_word('ada', 2)
        t.add_word('perl', 3)
        t.add_word('pascal', 4)
        t.add_word('php', 5)

        result = list(t.values())
        self.assertEqual(len(result), 5)
        self.assertIn(1, result)
        self.assertIn(2, result)
        self.assertIn(3, result)
        self.assertIn(4, result)
        self.assertIn(5, result)


    def testClearShouldRemoveEveryting(self):
        t = Trie()

        t.add_word('python', 1)
        t.add_word('ada', 2)
        t.add_word('perl', 3)
        t.add_word('pascal', 4)
        t.add_word('php', 5)

        self.assertEqual(len(t), 5)
        self.assertEqual(len(list(t.items())), 5)

        t.clear()

        self.assertEqual(len(t), 0)
        self.assertEqual(len(list(t.items())), 0)


    def testIterShouldMatchAllStrings(self):

        def get_test_automaton():
            words = "he her hers his she hi him man himan".split()

            t = Trie();
            for w in words:
                t.add_word(w, w)

            t.make_automaton()

            return t


        test_string = "he she himan"

        t = get_test_automaton()
        result = list(t.iter(test_string))

        # there are 5 matching positions
        self.assertEqual(len(result), 5)

        # result should have be valid, i.e. returned position and substring
        # must match substring from test string
        for end_index, strings in result:
            for s in strings:
                n = len(s)
                self.assertEqual(s, test_string[end_index - n + 1 : end_index + 1])


    def testFindAllShouldGetTheSameDataAsIter(self):

        def get_test_automaton():
            words = "he her hers his she hi him man himan".split()

            t = Trie();
            for w in words:
                t.add_word(w, w)

            t.make_automaton()

            return t

        find_all_arguments = []

        def find_all_callback(end_index, strings):
            find_all_arguments.append((end_index, strings))

        t = get_test_automaton()
        test_string = "he she himan"

        t.find_all(test_string, find_all_callback)

        result_items = list(t.iter(test_string))
        self.assertEqual(find_all_arguments, result_items)


if __name__ == '__main__':
    unittest.main()



"""
	Aho-Corasick string search algorithm.

	Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
	WWW       : http://0x80.pl
	License   : public domain
"""

import pyahocorasick


def exportdot(trie, file):

    def writeln(text=""):
        file.write(text + "\n")

    writeln("digraph ahocorasick {")

    def walk(node):
        queue = [node]
        while queue:
            node = queue.pop()
            yield node

            for child in node.children.itervalues():
                if child != node:
                    queue.append(child)

    nodes = list(walk(trie.root))

    # nodes
    for node in nodes:
        if node.output != pyahocorasick.nil:
            writeln("\tnode%d [shape=doublecircle, label=\"\"]" % id(node))
        else:
            writeln("\tnode%d [shape=circle, label=\"\"]" % id(node))

    # trie edges
    for node in nodes:
        for letter, child in node.children.iteritems():
            nodeid = id(node)
            destid = id(child)
            if destid == id(trie.root):
                # do not show self-links of root node created during make_automaton
                continue

            if letter.isalnum():
                label = letter
            else:
                label = '%02x' % ord(letter)

            writeln("\tnode%d -> node%d [label=\"%s\"]" % (nodeid, destid, label))

    # fail links
    for node in nodes:
        nodeid = id(node)
        failid = id(node.fail)

        if failid != pyahocorasick.nil:
            writeln("\tnode%d -> node%d [color=blue]" % (nodeid, failid))

    writeln("}")


if __name__ == '__main__':
    A = pyahocorasick.Trie()

    A.add_word("he", 0)
    A.add_word("her", 1)
    A.add_word("hers", 2)
    A.add_word("she", 3)
    A.add_word("cat", 4)
    A.add_word("shield", 5)

    with open('trie.dot', 'wt') as f:
        exportdot(A, f)

    A.make_automaton()

    with open('ahocorasick.dot', 'wt') as f:
        exportdot(A, f)



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""
import ahocorasick

from pytestingutils import conv


def test_issue133_iter_long_1():
    automaton = ahocorasick.Automaton()
    automaton.add_word(conv("b"), "b")
    automaton.add_word(conv("abc"), "abc")
    automaton.make_automaton()

    res = list(automaton.iter_long(conv("abb")))

    expected = [(1, "b"), (2, "b")]
    assert res == expected


def test_issue133_iter_long_2():
    automaton = ahocorasick.Automaton()
    for word in ["b", "c", "abd"]:
        converted = conv(word)
        automaton.add_word(converted, word)
    automaton.make_automaton()

    res = list(automaton.iter_long(conv("abc")))

    expected = [(1, "b"), (2, "c")]
    assert res == expected


def test_issue133_iter_long_3():
    automaton = ahocorasick.Automaton()
    for word in ["trimethoprim", "sulfamethoxazole", "meth"]:
        converted = conv(word)
        automaton.add_word(converted, word)
    automaton.make_automaton()

    res = list(automaton.iter_long(conv("sulfamethoxazole and trimethoprim")))

    expected = [(15, "sulfamethoxazole"), (32, "trimethoprim")]
    assert res == expected

def test_issue133_iter_long_4():
    automaton = ahocorasick.Automaton()
    for word in ["is", "this", "is this a dream?"]:
        converted = conv(word)
        automaton.add_word(converted, word)
    automaton.make_automaton()

    res = list(automaton.iter_long(conv("is this a test?")))

    expected = [(1, "is"), (6, "this")]
    assert res == expected

def test_issue133_iter_long_5():
    automaton = ahocorasick.Automaton()
    for word in ["th", "this", "is this a dream?"]:
        converted = conv(word)
        automaton.add_word(converted, word)
    automaton.make_automaton()

    res = list(automaton.iter_long(conv("is this a test?")))

    expected = [(6, "this")]
    assert res == expected



def test_issue133_iter_long_with_multibyte_characters():
    automaton = ahocorasick.Automaton()
    for word in ["知识产权", "国家知识产权局2"]:
        converted = conv(word)
        automaton.add_word(converted, word)
    automaton.make_automaton()

    res = list(automaton.iter_long(conv("国家知识产权局1")))
    if ahocorasick.unicode:
        expected = [(5, "知识产权")]
    else:
        # UTF-8-bytes
        expected = [(17, "知识产权")]
    assert res == expected



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import sys


def main():
    path = sys.argv[1]

    app = Application(path)
    if app.run():
        sys.exit(0)
    else:
        sys.exit(1)


class Application(object):

    def __init__(self, path):
        self.path = path

    def run(self):
        with open(self.path, 'rt') as f:
            lines = [line.rstrip() for line in f if line.rstrip()]
            errors = self.analyze(lines)

        return errors

    def analyze(self, lines):
        error_sep = '======================================================================'
        traceback_sep = '----------------------------------------------------------------------'

        index = 0
        result = True
        while True:
            try:
                index = lines.index(error_sep, index)
            except ValueError:
                break

            index += 1
            function = lines[index]
            index += 1
            start = lines.index(traceback_sep, index)
            end = lines.index(traceback_sep, start + 1)
            index = end + 1

            error = lines[end - 1]
            if error != 'MemoryError':
                print('%s: %s' % (function, error))
                result = False

        return result


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import pytest

import ahocorasick

from pytestingutils import on_windows


@pytest.mark.skipif(ahocorasick.unicode, reason="Run only with bytes build")
@pytest.mark.xfail(reason="Fails on Windows")
def test_basic_bytes():
    automaton = ahocorasick.Automaton()
    words = b"he e hers his she hi him man he".split()
    #         0 1    2   3   4  5   6   7  8
    for i, w in enumerate(words):
        automaton.add_word(w, (i, w))
    query = b"he rshershidamanza "
    #        01234567890123
    automaton.make_automaton()

    assert query[2:8] == b' rsher'
    results = list(automaton.iter(string=query, start=2, end=8))
    assert results == [(6, (4, b'she')), (6, (8, b'he')), (6, (1, b'e'))]

    res = []

    def callback(index, item):
        res.append(dict(index=index, item=item))

    assert query[2:11] == b' rshershi'
    automaton.find_all(query, callback, 2, 11)

    expected = [
        {'index': 6, 'item': (4, b'she')},
        {'index': 6, 'item': (8, b'he')},
        {'index': 6, 'item': (1, b'e')},
        {'index': 8, 'item': (2, b'hers')},
        {'index': 10, 'item': (5, b'hi')},
    ]

    assert res == expected


@pytest.mark.skipif(ahocorasick.unicode, reason="Run only with bytes build")
@pytest.mark.xfail(reason="Fails on everywhere")
def test_basic_items_keys_and_values_with_bytes_build():
    automaton = ahocorasick.Automaton()
    words = b"he e hers his she hi him man he".split()
    #         1  2 3    4   5   6  7   8   9
    for i, w in enumerate(words, 1):
        automaton.add_word(w, (i, w))

    expected_keys = [b'e', b'hers', b'his', b'she', b'hi', b'him', b'man', b'he', ]
    expected_values = [
        # the second addition munges this by design, like in a dict (1, b'he'),
        (2, b'e'),
        (3, b'hers'),
        (4, b'his'),
        (5, b'she'),
        (6, b'hi'),
        (7, b'him'),
        (8, b'man'),
        (9, b'he'),
    ]

    expected_items = [
        # the second addition munges this by design, like in a dict  (b'he', (1, b'he')),
        (b'e', (2, b'e')),
        (b'hers', (3, b'hers')),
        (b'his', (4, b'his')),
        (b'she', (5, b'she')),
        (b'hi', (6, b'hi')),
        (b'him', (7, b'him')),
        (b'man', (8, b'man')),
        (b'he', (9, b'he')),
    ]

    assert sorted(automaton.keys()) == sorted(expected_keys)
    assert sorted(automaton.values()) == sorted(expected_values)
    assert sorted(dict(automaton.items()).values()) == sorted(expected_values)
    assert sorted(dict(automaton.items()).keys()) == sorted(expected_keys)

    automaton.make_automaton()

    assert sorted(automaton.keys()) == sorted(expected_keys)
    assert sorted(automaton.values()) == sorted(expected_values)
    assert sorted(dict(automaton.items()).values()) == sorted(expected_values)
    assert sorted(dict(automaton.items()).keys()) == sorted(expected_keys)
    assert sorted(automaton.items()) == sorted(expected_items)


@pytest.mark.skipif(not ahocorasick.unicode, reason="Run only with unicode build")
@pytest.mark.xfail(on_windows, reason="Fails on Windows")
def test_basic_unicode():
    automaton = ahocorasick.Automaton()
    words = "he e hers his she hi him man he".split()
    #        0  1 2    3   4   5  6   7   8
    for i, w in enumerate(words):
        automaton.add_word(w, (i, w))
    query = "he rshershidamanza "
    #        01234567890123
    automaton.make_automaton()

    assert query[2:8] == ' rsher'
    results = list(automaton.iter(string=query, start=2, end=8))
    assert results == [(6, (4, 'she')), (6, (8, 'he')), (6, (1, 'e'))]

    res = []

    def callback(index, item):
        res.append(dict(index=index, item=item))

    assert query[2:11] == ' rshershi'
    automaton.find_all(query, callback, 2, 11)

    expected = [
        {'index': 6, 'item': (4, 'she')},
        {'index': 6, 'item': (8, 'he')},
        {'index': 6, 'item': (1, 'e')},
        {'index': 8, 'item': (2, 'hers')},
        {'index': 10, 'item': (5, 'hi')},
    ]

    assert res == expected


@pytest.mark.skipif(not ahocorasick.unicode, reason="Run only with unicode build")
@pytest.mark.xfail(on_windows, reason="Fails on Windows")
def test_basic_items_keys_and_values_with_unicode_build():
    automaton = ahocorasick.Automaton()
    words = 'he e hers his she hi him man he'.split()
    #         0 1    2   3   4  5   6   7  8
    for i, w in enumerate(words):
        automaton.add_word(w, (i, w))

    expected_keys = ['man', 'she', 'e', 'hi', 'him', 'his', 'he', 'hers']

    expected_values = [
        (7, 'man'),
        (4, 'she'),
        (1, 'e'),
        (5, 'hi'),
        (6, 'him'),
        (3, 'his'),
        (8, 'he'),
        (2, 'hers'),
    ]

    assert sorted(automaton.keys()) == sorted(expected_keys)
    assert sorted(automaton.values()) == sorted(expected_values)
    assert sorted(dict(automaton.items()).values()) == sorted(expected_values)
    assert sorted(dict(automaton.items()).keys()) == sorted(expected_keys)

    automaton.make_automaton()

    assert sorted(automaton.keys()) == sorted(expected_keys)
    assert sorted(automaton.values()) == sorted(expected_values)
    assert sorted(dict(automaton.items()).values()) == sorted(expected_values)
    assert sorted(dict(automaton.items()).keys()) == sorted(expected_keys)



# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import sys


def main():
    try:
        path = sys.argv[1]
    except IndexError:
        path = 'memory.dump'

    app = Application(path)
    app.run()


class Application(object):

    def __init__(self, path):
        self.path = path

    def run(self):
        with open(self.path, 'rt') as f:
            print(max(self.ids(f)))

    def ids(self, file):
        for i, line in enumerate(file):
            fields = line.split()
            if fields[0] == 'R':
                yield int(fields[1])


if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-

"""
    Aho-Corasick string search algorithm.

    Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
    WWW       : http://0x80.pl
    License   : public domain
"""

import tempfile
from pathlib import Path
from pickle import load, dump

from ahocorasick import Automaton

from pytestingutils import conv


def test_issue_50():
    auto = Automaton()
    auto.add_word(conv('abc'), 'abc')

    auto.add_word(conv('def'), 'def')
    tempdir = Path(tempfile.mkdtemp(prefix="ahocorasick-test-"))
    pickled = tempdir / 'automaton-wee.pickle'

    with open(pickled, 'wb') as dest:
        dump(auto, dest)

    with open(pickled, 'rb') as src:
        auto2 = load(src)

    assert list(auto.items()) == list(auto2.items())



# -*- coding: utf-8 -*-

"""
Aho-Corasick string search algorithm.

Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
WWW       : http://0x80.pl
License   : BSD-3-Clause (see LICENSE)
"""

import os

from setuptools import setup
from setuptools import Extension

# check for presence of environment variables
build_as_bytes = "AHOCORASICK_BYTES" in os.environ
build_as_unicode = "AHOCORASICK_UNICODE" in os.environ

if build_as_unicode and build_as_bytes:
    raise Exception(
        "Cannot build pyahocorasick both as bytes and unicode: "
        "only one of AHOCORASICK_UNICODE and AHOCORASICK_BYTES environment "
        "variable should be set."
    )

# we default to unicode
elif not build_as_unicode and not build_as_bytes:
    build_as_unicode = True

if build_as_unicode:
    macros = [("AHOCORASICK_UNICODE", "")]
else:
    macros = []


def get_long_description():
    import io
    with io.open("README.rst", encoding="UTF-8") as f:
        return f.read()


module = Extension(
    "ahocorasick",
    sources=[
        "src/pyahocorasick.c",
    ],
    define_macros=macros,
    depends=[
        "src/common.h",
        "src/Automaton.c",
        "src/Automaton.h",
        "src/Automaton_pickle.c",
        "src/AutomatonItemsIter.c",
        "src/AutomatonItemsIter.h",
        "src/AutomatonSearchIter.c",
        "src/AutomatonSearchIter.h",
        "src/AutomatonSearchIterLong.c",
        "src/AutomatonSearchIterLong.h",
        "src/trie.c",
        "src/trie.h",
        "src/slist.c",
        "src/utils.c",
        "src/trienode.c",
        "src/trienode.h",
        "src/msinttypes/stdint.h",
        "src/inline_doc.h",
        "src/pickle/pickle.h",
        "src/pickle/pickle_data.h",
        "src/pickle/pickle_data.c",
        "src/custompickle/custompickle.h",
        "src/custompickle/custompickle.c",
        "src/custompickle/pyhelpers.h",
        "src/custompickle/pyhelpers.c",
        "src/custompickle/save/automaton_save.h",
        "src/custompickle/save/automaton_save.c",
        "src/custompickle/save/savebuffer.h",
        "src/custompickle/save/savebuffer.c",
        "src/custompickle/load/module_automaton_load.h",
        "src/custompickle/load/module_automaton_load.c",
        "src/custompickle/load/loadbuffer.h",
        "src/custompickle/load/loadbuffer.c",
        "src/pycallfault/pycallfault.h",
        "src/pycallfault/pycallfault.c",
    ],
)

setup(
    name="pyahocorasick",
    version="2.3.1",
    ext_modules=[module],

    description=(
        "pyahocorasick is a fast and memory efficient library for exact or "
        "approximate multi-pattern string search.  With the ``ahocorasick.Automaton`` "
        "class, you can find multiple key string occurrences at once in some input "
        "text.  You can use it as a plain dict-like Trie or convert a Trie to an "
        "automaton for efficient Aho-Corasick search. And pickle to disk for easy "
        "reuse of large automatons. Implemented in C and tested on Python 3.6+. "
        "Works on Linux, macOS and Windows. BSD-3-Cause license."
    ),
    author="Wojciech Muła",
    author_email="wojciech_mula@poczta.onet.pl",
    maintainer="Wojciech Muła",
    maintainer_email="wojciech_mula@poczta.onet.pl",
    url="http://github.com/WojciechMula/pyahocorasick",
    platforms=["Linux", "MacOSX", "Windows"],
    license=" BSD-3-Clause and Public-Domain",
    long_description=get_long_description(),
    long_description_content_type="text/x-rst",
    keywords=[
        "aho-corasick",
        "trie",
        "automaton",
        "dictionary",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: C",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries",
        "Topic :: Text Editors :: Text Processing",
    ],
    extras_require={
        "testing": ["pytest", "twine", "setuptools", "wheel", ],
    },
    python_requires=">=3.10",
)



# -*- coding: utf-8 -*-
"""
This is part of pyahocorasick Python module.

Unit tests for the C-based ahocorasick module.

Author    : Wojciech Muła, wojciech_mula@poczta.onet.pl
WWW       : http://0x80.pl/proj/pyahocorasick/
License   : public domain
"""
import sys

import ahocorasick

if ahocorasick.unicode:
    conv = lambda x: x
else:
    conv = lambda x: bytes(x, encoding='utf-8')


sys_platform = str(sys.platform).lower()
on_linux = sys_platform.startswith('linux')
on_windows = 'win32' in sys_platform
on_mac = 'darwin' in sys_platform
on_freebsd = 'freebsd' in sys_platform
